"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.compareAndRemove = exports.transformCompareAndRemoveReply = exports.transformCompareAndRemoveArguments = exports.defineCompareAndRemoveCommand = void 0;
const fs = require("fs");
const path = require("path");
const keys_1 = require("../keys");
const definition = {
    numberOfKeys: 5,
    lua: fs.readFileSync(path.join(__dirname, 'compareAndRemove.lua'), 'utf-8'),
};
function defineCompareAndRemoveCommand(redis) {
    redis.defineCommand('lcCompareAndRemove', definition);
}
exports.defineCompareAndRemoveCommand = defineCompareAndRemoveCommand;
function transformCompareAndRemoveArguments(collection, key, compareOperator, compareVersion, maxlen = 1000) {
    return [
        keys_1.getKey(collection, keys_1.VALUES_HASH_SUFFIX),
        keys_1.getKey(collection, keys_1.KEYS_SORTED_SET_SUFFIX),
        keys_1.getKey(collection, keys_1.VERSIONS_SORTED_SET_SUFFIX),
        keys_1.getKey(collection, keys_1.CHANGES_STREAM_SUFFIX),
        keys_1.getKey(collection, keys_1.REVISION_SUFFIX),
        key,
        compareOperator,
        compareVersion.toString(),
        maxlen,
    ];
}
exports.transformCompareAndRemoveArguments = transformCompareAndRemoveArguments;
function transformCompareAndRemoveReply(reply) {
    const [revision, success] = reply;
    return {
        revision: revision.toString(),
        success: !!success,
    };
}
exports.transformCompareAndRemoveReply = transformCompareAndRemoveReply;
function compareAndRemove(redis, collection, key, compareOperator, compareVersion, maxlen = 1000) {
    return redis
        .lcCompareAndRemoveBuffer(...transformCompareAndRemoveArguments(collection, key, compareOperator, compareVersion, maxlen))
        .then(transformCompareAndRemoveReply);
}
exports.compareAndRemove = compareAndRemove;
//# sourceMappingURL=compareAndRemove.js.map