"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.compareAndSet = exports.transformCompareAndSetReply = exports.transformCompareAndSetArguments = exports.defineCompareAndSetCommand = void 0;
const fs = require("fs");
const path = require("path");
const keys_1 = require("../keys");
const definition = {
    numberOfKeys: 5,
    lua: fs.readFileSync(path.join(__dirname, 'compareAndSet.lua'), 'utf-8'),
};
function defineCompareAndSetCommand(redis) {
    redis.defineCommand('lcCompareAndSet', definition);
}
exports.defineCompareAndSetCommand = defineCompareAndSetCommand;
function transformCompareAndSetArguments(collection, key, compareOperator, compareVersion, value, version = Infinity, maxlen = 1000) {
    return [
        keys_1.getKey(collection, keys_1.VALUES_HASH_SUFFIX),
        keys_1.getKey(collection, keys_1.KEYS_SORTED_SET_SUFFIX),
        keys_1.getKey(collection, keys_1.VERSIONS_SORTED_SET_SUFFIX),
        keys_1.getKey(collection, keys_1.CHANGES_STREAM_SUFFIX),
        keys_1.getKey(collection, keys_1.REVISION_SUFFIX),
        key,
        compareOperator,
        compareVersion.toString(),
        value,
        version.toString(),
        maxlen,
    ];
}
exports.transformCompareAndSetArguments = transformCompareAndSetArguments;
function transformCompareAndSetReply(reply) {
    const [revision, success] = reply;
    return {
        revision: revision.toString(),
        success: !!success,
    };
}
exports.transformCompareAndSetReply = transformCompareAndSetReply;
function compareAndSet(redis, collection, key, compareOperator, compareVersion, value, version = Infinity, maxlen = 1000) {
    return redis
        .lcCompareAndSetBuffer(...transformCompareAndSetArguments(collection, key, compareOperator, compareVersion, value, version, maxlen))
        .then(transformCompareAndSetReply);
}
exports.compareAndSet = compareAndSet;
//# sourceMappingURL=compareAndSet.js.map