"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.get = exports.transformGetReply = exports.transformGetArguments = exports.defineGetCommand = void 0;
const fs = require("fs");
const path = require("path");
const keys_1 = require("../keys");
const versions_1 = require("../versions");
const definition = {
    numberOfKeys: 3,
    lua: fs.readFileSync(path.join(__dirname, 'get.lua'), 'utf-8'),
};
function defineGetCommand(redis) {
    redis.defineCommand('lcGet', definition);
}
exports.defineGetCommand = defineGetCommand;
function transformGetArguments(collection, key) {
    return [
        keys_1.getKey(collection, keys_1.VALUES_HASH_SUFFIX),
        keys_1.getKey(collection, keys_1.VERSIONS_SORTED_SET_SUFFIX),
        keys_1.getKey(collection, keys_1.REVISION_SUFFIX),
        key,
    ];
}
exports.transformGetArguments = transformGetArguments;
function transformGetReply(reply) {
    const [revision, value = null, version] = reply;
    return {
        revision: revision.toString(),
        value,
        version: version == null ? 0 : versions_1.versionFromString(version.toString()),
    };
}
exports.transformGetReply = transformGetReply;
function get(redis, collection, key) {
    return redis
        .lcGetBuffer(...transformGetArguments(collection, key))
        .then(transformGetReply);
}
exports.get = get;
//# sourceMappingURL=get.js.map