"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getAll = void 0;
const keys_1 = require("../keys");
const versions_1 = require("../versions");
async function getAll(redis, collection) {
    const [[valuesErr, valuesResult], [versionsErr, versionsResult], [revisionErr, revisionResult],] = await redis
        .multi()
        .hgetallBuffer(keys_1.getKey(collection, keys_1.VALUES_HASH_SUFFIX))
        .zrangeBuffer(keys_1.getKey(collection, keys_1.VERSIONS_SORTED_SET_SUFFIX), 0, -1, 'WITHSCORES')
        .get(keys_1.getKey(collection, keys_1.REVISION_SUFFIX))
        .exec();
    if (valuesErr) {
        throw valuesErr;
    }
    if (versionsErr) {
        throw versionsErr;
    }
    if (revisionErr) {
        throw revisionErr;
    }
    const items = [];
    for (let i = 0; i < versionsResult.length; i += 2) {
        const key = versionsResult[i];
        const version = versions_1.versionFromString(versionsResult[i + 1].toString());
        const value = valuesResult[key];
        items.push({
            key,
            version,
            value,
        });
    }
    return {
        revision: revisionResult !== null && revisionResult !== void 0 ? revisionResult : '0-0',
        items,
    };
}
exports.getAll = getAll;
//# sourceMappingURL=getAll.js.map