"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getKeyRange = exports.transformGetKeyRangeReply = exports.transformGetKeyRangeArguments = exports.defineGetKeyRangeCommand = void 0;
const fs = require("fs");
const path = require("path");
const keys_1 = require("../keys");
const versions_1 = require("../versions");
const definition = {
    numberOfKeys: 4,
    lua: fs.readFileSync(path.join(__dirname, 'getKeyRange.lua'), 'utf-8'),
};
function defineGetKeyRangeCommand(redis) {
    redis.defineCommand('lcGetKeyRange', definition);
}
exports.defineGetKeyRangeCommand = defineGetKeyRangeCommand;
function transformGetKeyRangeArguments(collection, min, max) {
    return [
        keys_1.getKey(collection, keys_1.VALUES_HASH_SUFFIX),
        keys_1.getKey(collection, keys_1.KEYS_SORTED_SET_SUFFIX),
        keys_1.getKey(collection, keys_1.VERSIONS_SORTED_SET_SUFFIX),
        keys_1.getKey(collection, keys_1.REVISION_SUFFIX),
        min,
        max,
    ];
}
exports.transformGetKeyRangeArguments = transformGetKeyRangeArguments;
function transformGetKeyRangeReply(reply) {
    const [revision, items] = reply;
    return {
        revision: revision.toString(),
        items: items.map(([key, value, version]) => ({
            key,
            value,
            version: versions_1.versionFromString(version.toString()),
        })),
    };
}
exports.transformGetKeyRangeReply = transformGetKeyRangeReply;
function getKeyRange(redis, collection, min, max) {
    return redis
        .lcGetKeyRangeBuffer(...transformGetKeyRangeArguments(collection, min, max))
        .then(transformGetKeyRangeReply);
}
exports.getKeyRange = getKeyRange;
//# sourceMappingURL=getKeyRange.js.map