"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getVersionRange = exports.transformGetVersionRangeReply = exports.transformGetVersionRangeArguments = exports.defineGetVersionRangeCommand = void 0;
const fs = require("fs");
const path = require("path");
const keys_1 = require("../keys");
const versions_1 = require("../versions");
const definition = {
    numberOfKeys: 3,
    lua: fs.readFileSync(path.join(__dirname, 'getVersionRange.lua'), 'utf-8'),
};
function defineGetVersionRangeCommand(redis) {
    redis.defineCommand('lcGetVersionRange', definition);
}
exports.defineGetVersionRangeCommand = defineGetVersionRangeCommand;
function transformGetVersionRangeArguments(collection, min, max) {
    return [
        keys_1.getKey(collection, keys_1.VALUES_HASH_SUFFIX),
        keys_1.getKey(collection, keys_1.VERSIONS_SORTED_SET_SUFFIX),
        keys_1.getKey(collection, keys_1.REVISION_SUFFIX),
        typeof min === 'string' ? min : versions_1.versionToString(min),
        typeof max === 'string' ? max : versions_1.versionToString(max),
    ];
}
exports.transformGetVersionRangeArguments = transformGetVersionRangeArguments;
function transformGetVersionRangeReply(reply) {
    const [revision, items] = reply;
    return {
        revision: revision.toString(),
        items: items.map(([key, value, version]) => ({
            key,
            value,
            version: versions_1.versionFromString(version.toString()),
        })),
    };
}
exports.transformGetVersionRangeReply = transformGetVersionRangeReply;
function getVersionRange(redis, collection, min, max) {
    return redis
        .lcGetVersionRangeBuffer(...transformGetVersionRangeArguments(collection, min, max))
        .then(transformGetVersionRangeReply);
}
exports.getVersionRange = getVersionRange;
//# sourceMappingURL=getVersionRange.js.map