"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.removeKeyRange = exports.transformRemoveKeyRangeReply = exports.transformRemoveKeyRangeArguments = exports.defineRemoveKeyRangeCommand = void 0;
const fs = require("fs");
const path = require("path");
const keys_1 = require("../keys");
const definition = {
    numberOfKeys: 5,
    lua: fs.readFileSync(path.join(__dirname, 'removeKeyRange.lua'), 'utf-8'),
};
function defineRemoveKeyRangeCommand(redis) {
    redis.defineCommand('lcRemoveKeyRange', definition);
}
exports.defineRemoveKeyRangeCommand = defineRemoveKeyRangeCommand;
function transformRemoveKeyRangeArguments(collection, min, max, maxlen = 1000) {
    return [
        keys_1.getKey(collection, keys_1.VALUES_HASH_SUFFIX),
        keys_1.getKey(collection, keys_1.KEYS_SORTED_SET_SUFFIX),
        keys_1.getKey(collection, keys_1.VERSIONS_SORTED_SET_SUFFIX),
        keys_1.getKey(collection, keys_1.CHANGES_STREAM_SUFFIX),
        keys_1.getKey(collection, keys_1.REVISION_SUFFIX),
        min,
        max,
        maxlen,
    ];
}
exports.transformRemoveKeyRangeArguments = transformRemoveKeyRangeArguments;
function transformRemoveKeyRangeReply(reply) {
    const [revision, removedCount] = reply;
    return {
        revision: revision.toString(),
        removedCount,
    };
}
exports.transformRemoveKeyRangeReply = transformRemoveKeyRangeReply;
function removeKeyRange(redis, collection, min, max, maxlen = 1000) {
    return redis
        .lcRemoveKeyRangeBuffer(...transformRemoveKeyRangeArguments(collection, min, max, maxlen))
        .then(transformRemoveKeyRangeReply);
}
exports.removeKeyRange = removeKeyRange;
//# sourceMappingURL=removeKeyRange.js.map