"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.removeVersionRange = exports.transformRemoveVersionRangeReply = exports.transformRemoveVersionRangeArguments = exports.defineRemoveVersionRangeCommand = void 0;
const fs = require("fs");
const path = require("path");
const keys_1 = require("../keys");
const versions_1 = require("../versions");
const definition = {
    numberOfKeys: 5,
    lua: fs.readFileSync(path.join(__dirname, 'removeVersionRange.lua'), 'utf-8'),
};
function defineRemoveVersionRangeCommand(redis) {
    redis.defineCommand('lcRemoveVersionRange', definition);
}
exports.defineRemoveVersionRangeCommand = defineRemoveVersionRangeCommand;
function transformRemoveVersionRangeArguments(collection, min, max, maxlen = 1000) {
    return [
        keys_1.getKey(collection, keys_1.VALUES_HASH_SUFFIX),
        keys_1.getKey(collection, keys_1.KEYS_SORTED_SET_SUFFIX),
        keys_1.getKey(collection, keys_1.VERSIONS_SORTED_SET_SUFFIX),
        keys_1.getKey(collection, keys_1.CHANGES_STREAM_SUFFIX),
        keys_1.getKey(collection, keys_1.REVISION_SUFFIX),
        typeof min === 'string' ? min : versions_1.versionToString(min),
        typeof max === 'string' ? max : versions_1.versionToString(max),
        maxlen,
    ];
}
exports.transformRemoveVersionRangeArguments = transformRemoveVersionRangeArguments;
function transformRemoveVersionRangeReply(reply) {
    const [revision, removedCount] = reply;
    return {
        revision: revision.toString(),
        removedCount,
    };
}
exports.transformRemoveVersionRangeReply = transformRemoveVersionRangeReply;
function removeVersionRange(redis, collection, min, max, maxlen = 1000) {
    return redis
        .lcRemoveVersionRangeBuffer(...transformRemoveVersionRangeArguments(collection, min, max, maxlen))
        .then(transformRemoveVersionRangeReply);
}
exports.removeVersionRange = removeVersionRange;
//# sourceMappingURL=removeVersionRange.js.map