"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.set = exports.transformSetReply = exports.transformSetArguments = exports.defineSetCommand = void 0;
const fs = require("fs");
const path = require("path");
const keys_1 = require("../keys");
const versions_1 = require("../versions");
const definition = {
    numberOfKeys: 5,
    lua: fs.readFileSync(path.join(__dirname, 'set.lua'), 'utf-8'),
};
function defineSetCommand(redis) {
    redis.defineCommand('lcSet', definition);
}
exports.defineSetCommand = defineSetCommand;
function transformSetArguments(collection, key, value, version = Infinity, maxlen = 1000) {
    return [
        keys_1.getKey(collection, keys_1.VALUES_HASH_SUFFIX),
        keys_1.getKey(collection, keys_1.KEYS_SORTED_SET_SUFFIX),
        keys_1.getKey(collection, keys_1.VERSIONS_SORTED_SET_SUFFIX),
        keys_1.getKey(collection, keys_1.CHANGES_STREAM_SUFFIX),
        keys_1.getKey(collection, keys_1.REVISION_SUFFIX),
        key,
        value,
        versions_1.versionToString(version),
        maxlen,
    ];
}
exports.transformSetArguments = transformSetArguments;
function transformSetReply(reply) {
    const [revision] = reply;
    return {
        revision: revision.toString(),
    };
}
exports.transformSetReply = transformSetReply;
function set(redis, collection, key, value, version = Infinity, maxlen = 1000) {
    return redis
        .lcSetBuffer(...transformSetArguments(collection, key, value, version, maxlen))
        .then(transformSetReply);
}
exports.set = set;
//# sourceMappingURL=set.js.map