"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.watch = void 0;
const rxjs_1 = require("rxjs");
const changeEvents_1 = require("../changeEvents");
const keys_1 = require("../keys");
function watch(redis, collection, lastRevision, blockMs = 2500) {
    return new rxjs_1.Observable((subscriber) => {
        async function iterate() {
            while (true) {
                if (subscriber.closed) {
                    return;
                }
                const result = await redis.xreadBuffer('BLOCK', blockMs, 'STREAMS', keys_1.getKey(collection, keys_1.CHANGES_STREAM_SUFFIX), lastRevision);
                if (result == null) {
                    continue;
                }
                const entries = result[0][1];
                const events = entries.map((entry) => changeEvents_1.changeEventFromEntry(entry));
                if (events.length > 0) {
                    if (events[0].prevRevision !== lastRevision) {
                        throw new Error('Failed to continuously read change events from stream. Consider increasing `maxlen` parameter');
                    }
                    lastRevision = events[events.length - 1].revision;
                }
                subscriber.next(events);
            }
        }
        iterate().catch((err) => subscriber.error(err));
    });
}
exports.watch = watch;
//# sourceMappingURL=watch.js.map